import { Button } from '@highlight-ui/button';
import { Icon } from '@highlight-ui/icon';
import { parseMetadata } from '@highlight-ui/utils-commons';
import classnames from 'classnames';
import MonthSelectPlugin from 'flatpickr/dist/plugins/monthSelect';
import * as React from 'react';
import Flatpickr from 'flatpickr';
import { German } from 'flatpickr/dist/l10n/de';
import { english } from 'flatpickr/dist/l10n/default';
import { Spanish } from 'flatpickr/dist/l10n/es';
import { French } from 'flatpickr/dist/l10n/fr';
import { Dutch } from 'flatpickr/dist/l10n/nl';
import { Italian } from 'flatpickr/dist/l10n/it';

/*! *****************************************************************************
Copyright (c) Microsoft Corporation.

Permission to use, copy, modify, and/or distribute this software for any
purpose with or without fee is hereby granted.

THE SOFTWARE IS PROVIDED "AS IS" AND THE AUTHOR DISCLAIMS ALL WARRANTIES WITH
REGARD TO THIS SOFTWARE INCLUDING ALL IMPLIED WARRANTIES OF MERCHANTABILITY
AND FITNESS. IN NO EVENT SHALL THE AUTHOR BE LIABLE FOR ANY SPECIAL, DIRECT,
INDIRECT, OR CONSEQUENTIAL DAMAGES OR ANY DAMAGES WHATSOEVER RESULTING FROM
LOSS OF USE, DATA OR PROFITS, WHETHER IN AN ACTION OF CONTRACT, NEGLIGENCE OR
OTHER TORTIOUS ACTION, ARISING OUT OF OR IN CONNECTION WITH THE USE OR
PERFORMANCE OF THIS SOFTWARE.
***************************************************************************** */
/* global Reflect, Promise */

var extendStatics = function(d, b) {
    extendStatics = Object.setPrototypeOf ||
        ({ __proto__: [] } instanceof Array && function (d, b) { d.__proto__ = b; }) ||
        function (d, b) { for (var p in b) if (Object.prototype.hasOwnProperty.call(b, p)) d[p] = b[p]; };
    return extendStatics(d, b);
};

function __extends(d, b) {
    if (typeof b !== "function" && b !== null)
        throw new TypeError("Class extends value " + String(b) + " is not a constructor or null");
    extendStatics(d, b);
    function __() { this.constructor = d; }
    d.prototype = b === null ? Object.create(b) : (__.prototype = b.prototype, new __());
}

var __assign = function() {
    __assign = Object.assign || function __assign(t) {
        for (var s, i = 1, n = arguments.length; i < n; i++) {
            s = arguments[i];
            for (var p in s) if (Object.prototype.hasOwnProperty.call(s, p)) t[p] = s[p];
        }
        return t;
    };
    return __assign.apply(this, arguments);
};

function __rest(s, e) {
    var t = {};
    for (var p in s) if (Object.prototype.hasOwnProperty.call(s, p) && e.indexOf(p) < 0)
        t[p] = s[p];
    if (s != null && typeof Object.getOwnPropertySymbols === "function")
        for (var i = 0, p = Object.getOwnPropertySymbols(s); i < p.length; i++) {
            if (e.indexOf(p[i]) < 0 && Object.prototype.propertyIsEnumerable.call(s, p[i]))
                t[p[i]] = s[p[i]];
        }
    return t;
}

var TEST_IDS = {
    MONTH_PICKER_INPUT: 'month-picker-input',
    MONTH_PICKER_TODAY_BUTTON: 'month-picker-today-button',
    MONTH_PICKER_PREVIOUS_BUTTON: 'month-picker-previous-button',
    MONTH_PICKER_NEXT_BUTTON: 'month-picker-next-button',
};

var styles = {"wrapper":"MonthNavigation-module__2d3tHl-2__v1-1-11","flexCenter":"MonthNavigation-module__34KRl-Z1__v1-1-11","navigationButton":"MonthNavigation-module__20hvZLD7__v1-1-11","input":"MonthNavigation-module__112VBeMZ__v1-1-11","small":"MonthNavigation-module__2a8J1BcR__v1-1-11","medium":"MonthNavigation-module__1RldgTqk__v1-1-11","todayButton":"MonthNavigation-module__3vpdg_O9__v1-1-11","icon":"MonthNavigation-module__1vwd0lYL__v1-1-11"};

var MonthNavigationInput = /** @class */ (function (_super) {
    __extends(MonthNavigationInput, _super);
    function MonthNavigationInput() {
        var _this = _super !== null && _super.apply(this, arguments) || this;
        _this.ref = null;
        _this.handleRef = function (ref) {
            if (!ref)
                return;
            _this.ref = ref;
        };
        return _this;
    }
    MonthNavigationInput.prototype.render = function () {
        return React.createElement("input", __assign({}, this.props, { ref: this.handleRef }));
    };
    return MonthNavigationInput;
}(React.Component));

var defaultOptions = {
    locale: 'en-GB',
    initialDate: undefined,
    disableCloseOnSelectDate: false,
    disableCloseOnClickOutside: false,
    disableToggleOnClick: false,
    title: '',
    onTitleClick: function () { },
};
var defaultFlatPickrOption = {
    enableTime: false,
    inline: true,
    allowInput: false,
    prevArrow: '<i class="fas fa-chevron-left"></i>',
    nextArrow: '<i class="fas fa-chevron-right"></i>',
};
var POSITION = {
    BOTTOM: 'bottom',
    TOP: 'top',
};

var transformDateFormat = function (dateFormat) {
    if (typeof dateFormat === 'string') {
        return dateFormat
            .replace('DD', 'd')
            .replace('MM', 'm')
            .replace('YYYY', 'Y')
            .replace('YY', 'y');
    }
    return dateFormat;
};
var customEnglish = __assign(__assign({}, english), { firstDayOfWeek: 1 });
var localeDictionary = {
    'en-GB': {
        locale: customEnglish,
    },
    'de-DE': {
        locale: German,
    },
    'es-ES': {
        locale: Spanish,
    },
    'fr-FR': {
        locale: French,
    },
    'nl-NL': {
        locale: Dutch,
    },
    'it-IT': {
        locale: Italian,
    },
};

var DatePicker = /** @class */ (function () {
    function DatePicker(node, options) {
        var _this = this;
        this._createCalendarWrapper = function () {
            var wrapper = document.createElement('div');
            wrapper.classList.add('personio-datepicker-wrapper');
            wrapper.appendChild(document.createElement('div'));
            if (wrapper.firstChild) {
                wrapper.firstChild.appendChild(document.createElement('div'));
            }
            return wrapper;
        };
        this._findNodeParent = function () {
            var wrapper = _this._node.parentNode;
            wrapper.classList.add('personio-datepicker-input-wrapper');
            return wrapper;
        };
        this._createTitle = function (str, onTitleClick, wrapper) {
            var title = document.createElement('span');
            title.textContent = str;
            title.classList.add('personio-datepicker-wrapper-title');
            if (title.textContent !== '') {
                title.classList.add('padding-bottom');
            }
            title.addEventListener('click', onTitleClick);
            wrapper.insertBefore(title, wrapper.firstChild);
            return title;
        };
        this._setCalendarWrapperPosition = function () {
            if (_this._position === POSITION.BOTTOM) {
                return;
            }
            var calendarWrapperInfo = _this._calendarWrapper.getBoundingClientRect();
            var nodeParentInfo = _this._nodeParent.getBoundingClientRect();
            if (_this._position === POSITION.TOP) {
                _this._calendarWrapper.style.top = '';
                _this._calendarWrapper.style.bottom = nodeParentInfo.height + "px";
                return;
            }
            if (calendarWrapperInfo.bottom >= window.innerHeight) {
                _this._calendarWrapper.style.top = '';
                _this._calendarWrapper.style.bottom = nodeParentInfo.height + "px";
            }
        };
        this._resetCalendarWrapperPosition = function () {
            var nodeParentInfo = _this._nodeParent.getBoundingClientRect();
            _this._calendarWrapper.style.top = nodeParentInfo.height + "px";
            _this._calendarWrapper.style.bottom = '';
        };
        this._addEventListeners = function () {
            if (!_this._option.disableToggleOnClick) {
                _this._node.addEventListener('click', _this._openDatePicker);
                _this._node.addEventListener('focus', _this._openDatePicker);
            }
            if (!_this._option.disableCloseOnClickOutside) {
                window.document.addEventListener('click', _this._clickOutsideHandler);
            }
            _this._title.addEventListener('click', _this._onTitleClick);
        };
        this._clickOutsideHandler = function (_a) {
            var target = _a.target;
            var targetElement = target;
            do {
                if (_this._node === targetElement ||
                    _this._calendarWrapper === targetElement) {
                    return;
                }
                // typecasting to prevent flow type checking error
                targetElement = targetElement.parentNode;
            } while (targetElement);
            _this._closeDatePicker();
        };
        this._openDatePicker = function () {
            _this._calendarWrapper.classList.add('show');
            _this._setCalendarWrapperPosition();
            if (_this._node !== document.activeElement) {
                _this._node.focus();
            }
        };
        this._closeDatePicker = function () {
            _this._calendarWrapper.classList.remove('show');
            _this._resetCalendarWrapperPosition();
        };
        this.toggleDatePicker = function () {
            if (_this._calendarWrapper.classList.contains('show')) {
                _this._closeDatePicker();
            }
            else {
                _this._openDatePicker();
            }
        };
        this.destroy = function () {
            if (!_this._option.disableToggleOnClick) {
                _this._node.removeEventListener('click', _this._openDatePicker);
                _this._node.removeEventListener('focus', _this._openDatePicker);
            }
            if (!_this._option.disableCloseOnClickOutside) {
                window.document.removeEventListener('click', _this._clickOutsideHandler);
            }
            _this._title.removeEventListener('click', _this._onTitleClick);
            _this._flatPickrInstance.destroy();
            _this._nodeParent.removeChild(_this._calendarWrapper);
        };
        var optionsWithDefault = __assign(__assign({}, defaultOptions), options);
        if (node) {
            var initialDate = optionsWithDefault.initialDate, locale = optionsWithDefault.locale, onChange_1 = optionsWithDefault.onChange, disableCloseOnSelectDate_1 = optionsWithDefault.disableCloseOnSelectDate, disableCloseOnClickOutside = optionsWithDefault.disableCloseOnClickOutside, title = optionsWithDefault.title, onTitleClick = optionsWithDefault.onTitleClick, disableToggleOnClick = optionsWithDefault.disableToggleOnClick, position = optionsWithDefault.position, restOption = __rest(optionsWithDefault, ["initialDate", "locale", "onChange", "disableCloseOnSelectDate", "disableCloseOnClickOutside", "title", "onTitleClick", "disableToggleOnClick", "position"]);
            this._node = node;
            this._nodeParent = this._findNodeParent();
            this._calendarWrapper = this._createCalendarWrapper();
            this._nodeParent.appendChild(this._calendarWrapper);
            this._onTitleClick = onTitleClick;
            this._title = this._createTitle(title, this._onTitleClick, this._calendarWrapper);
            var onChangeModified = function (dateArray) {
                if (!disableCloseOnSelectDate_1) {
                    _this._closeDatePicker();
                }
                var date = Array.isArray(dateArray) ? dateArray[0] : undefined;
                onChange_1(date);
            };
            var finalOption = __assign(__assign(__assign(__assign({}, defaultFlatPickrOption), restOption), localeDictionary[locale]), { onChange: onChangeModified });
            this._option = {
                disableToggleOnClick: disableToggleOnClick,
                disableCloseOnClickOutside: disableCloseOnClickOutside,
            };
            // eslint-disable-next-line @typescript-eslint/ban-ts-comment
            // @ts-ignore Flatpicker does not have a "void" return type
            this._flatPickrInstance = new Flatpickr(this._calendarWrapper.firstChild, finalOption);
            if (initialDate instanceof Date) {
                this._flatPickrInstance.setDate(initialDate);
            }
            this._position = position;
            this._addEventListeners();
        }
    }
    return DatePicker;
}());

var ReactDatePickerRaw = /** @class */ (function (_super) {
    __extends(ReactDatePickerRaw, _super);
    function ReactDatePickerRaw(props) {
        var _this = _super.call(this, props) || this;
        _this.inputRef = React.createRef();
        _this.setDateOnCalendarAndInput = function (date) {
            _this.setDateValueWithFormat(date);
            var TRIGGER_ONCHANGE_FOR_PLUGINS = true;
            _this.datePicker._flatPickrInstance.setDate(date, TRIGGER_ONCHANGE_FOR_PLUGINS);
        };
        _this.setDateValueWithFormat = function (date) {
            var dateFormat = _this.props.dateFormat;
            var transformedDateFormat = transformDateFormat(dateFormat);
            _this.setState({
                value: _this.datePicker._flatPickrInstance.formatDate(date, transformedDateFormat),
            });
        };
        _this.onInputChangeHandler = function (e) {
            if (e.target instanceof HTMLInputElement) {
                _this.setState({
                    value: e.target.value,
                }, function () {
                    var value = _this.state.value;
                    var _a = _this.props, dateFormat = _a.dateFormat, onDateChange = _a.onDateChange;
                    var isValidDate = value.length === (dateFormat ? dateFormat.length : -1);
                    var transformedDateFormat = transformDateFormat(dateFormat);
                    var parsedDate = _this.datePicker._flatPickrInstance.parseDate(value, transformedDateFormat);
                    var validatedDate = isValidDate ? parsedDate : undefined;
                    onDateChange(validatedDate);
                    _this.datePicker._flatPickrInstance.setDate(validatedDate);
                });
            }
        };
        _this.onDateChangeHandler = function (date) {
            var onDateChange = _this.props.onDateChange;
            _this.setDateValueWithFormat(date);
            onDateChange(date);
        };
        _this.renderDatePicker = function () {
            var _a = _this.props, flatpickrOptions = _a.flatpickrOptions, locale = _a.locale, initialDate = _a.initialDate, disableCloseOnSelectDate = _a.disableCloseOnSelectDate, disableCloseOnClickOutside = _a.disableCloseOnClickOutside, disableToggleOnClick = _a.disableToggleOnClick, title = _a.title, onTitleClick = _a.onTitleClick, position = _a.position;
            var options = __assign(__assign({}, flatpickrOptions), { locale: locale, initialDate: initialDate, title: title, onTitleClick: onTitleClick, disableCloseOnSelectDate: disableCloseOnSelectDate, disableCloseOnClickOutside: disableCloseOnClickOutside, disableToggleOnClick: disableToggleOnClick, position: position, onChange: _this.onDateChangeHandler });
            if (_this.datePicker) {
                _this.datePicker.destroy();
            }
            // eslint-disable-next-line @typescript-eslint/ban-ts-comment
            // @ts-ignore migrate the LegacyRef so it will be type checked
            _this.datePicker = new DatePicker(_this.inputRef.current.ref, options);
        };
        _this.state = {
            value: '',
        };
        return _this;
    }
    ReactDatePickerRaw.prototype.componentDidMount = function () {
        this.renderDatePicker();
        var _a = this.props, initialDate = _a.initialDate, selected = _a.selected;
        if (initialDate instanceof Date) {
            this.setDateValueWithFormat(initialDate);
        }
        if (selected instanceof Date) {
            this.setDateOnCalendarAndInput(selected);
        }
    };
    ReactDatePickerRaw.prototype.componentDidUpdate = function (_a) {
        var prevSelected = _a.selected;
        var selected = this.props.selected;
        if ((selected instanceof Date && typeof prevSelected === 'undefined') ||
            (selected instanceof Date &&
                prevSelected instanceof Date &&
                selected.toString() !== prevSelected.toString())) {
            this.setDateOnCalendarAndInput(selected);
        }
    };
    ReactDatePickerRaw.prototype.componentWillUnmount = function () {
        this.datePicker.destroy();
    };
    ReactDatePickerRaw.prototype.render = function () {
        var children = this.props.children;
        var value = this.state.value;
        return children({
            value: value,
            inputRef: this.inputRef,
            onChange: this.onInputChangeHandler,
        });
    };
    // eslint-disable-next-line react/static-property-placement
    ReactDatePickerRaw.defaultProps = {
        locale: 'en-GB',
        selected: undefined,
        initialDate: undefined,
        position: undefined,
        disableCloseOnSelectDate: false,
        disableCloseOnClickOutside: false,
        disableToggleOnClick: false,
        dateFormat: 'DD.MM.YYYY',
        title: '',
        onTitleClick: function () { },
        flatpickrOptions: {},
    };
    return ReactDatePickerRaw;
}(React.Component));

var MEDIUM = 'medium';
var SMALL = 'small';
var SIZE_OFFSET = 0;
function getIconSize(size) {
    if (size === SMALL) {
        return 18;
    }
    return 24;
}
function getSize(size) {
    var _a;
    return _a = {},
        _a[styles.small] = size === SMALL,
        _a[styles.medium] = size === MEDIUM,
        _a;
}
function notCurrentMonth(date) {
    return date.getMonth() !== new Date().getMonth();
}
var MonthNavigation = /** @class */ (function (_super) {
    __extends(MonthNavigation, _super);
    function MonthNavigation() {
        var _this = _super !== null && _super.apply(this, arguments) || this;
        // eslint-disable-next-line react/state-in-constructor
        _this.state = {
            date: new Date(),
        };
        _this.handleDateChange = function (date) {
            var onDateChange = _this.props.onDateChange;
            _this.setState({ date: date });
            onDateChange(date);
        };
        _this.decreaseDate = function () {
            var onDateChange = _this.props.onDateChange;
            var date = _this.state.date;
            if (date) {
                var newDate = new Date(date.getTime());
                newDate.setMonth(newDate.getMonth() - 1);
                _this.setState({ date: newDate });
                onDateChange(newDate);
            }
        };
        _this.increaseDate = function () {
            var onDateChange = _this.props.onDateChange;
            var date = _this.state.date;
            if (date) {
                var newDate = new Date(date.getTime());
                newDate.setMonth(newDate.getMonth() + 1);
                _this.setState({ date: newDate });
                onDateChange(newDate);
            }
        };
        _this.onTodayClick = function (e) {
            var onDateChange = _this.props.onDateChange;
            var date = new Date();
            _this.setState({ date: date });
            onDateChange(date);
            window.document.activeElement.blur();
            if (e) {
                e.preventDefault();
            }
        };
        return _this;
    }
    MonthNavigation.prototype.componentDidMount = function () {
        var initialDate = this.props.initialDate;
        if (initialDate) {
            this.setState({ date: initialDate });
        }
    };
    MonthNavigation.prototype.componentDidUpdate = function (_a) {
        var prevSelected = _a.selected;
        var selected = this.props.selected;
        if ((selected instanceof Date && typeof prevSelected === 'undefined') ||
            (selected instanceof Date &&
                prevSelected instanceof Date &&
                selected.toString() !== prevSelected.toString())) {
            // eslint-disable-next-line react/no-did-update-set-state
            this.setState({ date: selected });
        }
    };
    MonthNavigation.prototype.render = function () {
        var _this = this;
        var _a = this.props, size = _a.size, onFocus = _a.onFocus, onBlur = _a.onBlur, metadata = _a.metadata, showTodayButton = _a.showTodayButton, todayText = _a.todayText, 
        // datepicker props
        dateFormat = _a.dateFormat, flatpickrOptions = _a.flatpickrOptions, locale = _a.locale, initialDate = _a.initialDate, disableCloseOnSelectDate = _a.disableCloseOnSelectDate, disableCloseOnClickOutside = _a.disableCloseOnClickOutside, disableToggleOnClick = _a.disableToggleOnClick, monthSelectPluginOptions = _a.monthSelectPluginOptions;
        var date = this.state.date;
        return (React.createElement("div", null,
            React.createElement(ReactDatePickerRaw, { onDateChange: this.handleDateChange, dateFormat: dateFormat, selected: date, locale: locale, initialDate: initialDate, disableCloseOnSelectDate: disableCloseOnSelectDate, disableCloseOnClickOutside: disableCloseOnClickOutside, disableToggleOnClick: disableToggleOnClick, flatpickrOptions: __assign(__assign({}, flatpickrOptions), { plugins: [MonthSelectPlugin(__assign({}, monthSelectPluginOptions))] }) }, function (_a) {
                var value = _a.value, inputRef = _a.inputRef, onChange = _a.onChange;
                return (React.createElement("div", __assign({}, parseMetadata(metadata), { className: styles.wrapper }),
                    React.createElement("span", null,
                        React.createElement("button", { type: "button", onClick: _this.decreaseDate, className: styles.navigationButton, "data-test-id": TEST_IDS.MONTH_PICKER_PREVIOUS_BUTTON },
                            React.createElement(Icon, { name: "angle-left", size: getIconSize(size), className: styles.icon })),
                        React.createElement("button", { type: "button", onClick: _this.increaseDate, className: styles.navigationButton, "data-test-id": TEST_IDS.MONTH_PICKER_NEXT_BUTTON },
                            React.createElement(Icon, { name: "angle-right", size: getIconSize(size), className: styles.icon })),
                        React.createElement(MonthNavigationInput
                        // eslint-disable-next-line @typescript-eslint/ban-ts-comment
                        // @ts-ignore refactoring ReactDatePickerRaw not to use LegacyRef is needed
                        , { 
                            // eslint-disable-next-line @typescript-eslint/ban-ts-comment
                            // @ts-ignore refactoring ReactDatePickerRaw not to use LegacyRef is needed
                            ref: inputRef, value: value, type: "text", onChange: onChange, onFocus: onFocus, onBlur: onBlur, size: value.length + SIZE_OFFSET, className: classnames(styles.input, getSize(size)), "data-test-id": TEST_IDS.MONTH_PICKER_INPUT })),
                    showTodayButton && notCurrentMonth(date) && (React.createElement(Button, { onClick: _this.onTodayClick, theme: "tertiary", className: classnames(styles.todayButton, getSize(size)), "data-test-id": TEST_IDS.MONTH_PICKER_TODAY_BUTTON }, todayText))));
            })));
    };
    // eslint-disable-next-line react/static-property-placement
    MonthNavigation.defaultProps = {
        size: MEDIUM,
        locale: 'en-GB',
        selected: undefined,
        metadata: undefined,
        initialDate: undefined,
        onFocus: undefined,
        onBlur: undefined,
        disableCloseOnSelectDate: false,
        disableCloseOnClickOutside: false,
        disableToggleOnClick: false,
        dateFormat: 'F YYYY',
        flatpickrOptions: {},
        monthSelectPluginOptions: { shorthand: true },
        showTodayButton: true,
        todayText: 'Today',
    };
    return MonthNavigation;
}(React.Component));

export { MonthNavigation };
//# sourceMappingURL=index.js.map
